# -*- coding: utf-8 -*-

"""
    This script saves in DB the info from geojson files.
    Receives as parameter the path where geojson files are located.

    Usage:
    $ cd GeoInt_SIDT
    $ python manage.py populate_db [path/of/geojson/files]

    Example:
    $ cd GeoInt_SIDT
    $ python manage.py populate_db /home/emmanuelhp/Documentos/geojson/
"""

from django.core.management.base import BaseCommand, CommandError
from catalog.models import Polygon
import json, unicodedata
from shapely.geometry import shape


class Command(BaseCommand):

    def add_arguments(self, parser):
        ## positional arguments
        parser.add_argument('geojson_path', nargs=1, type=str)


    def handle(self, *args, **options):
        ## base geojson files path
        base_path = options['geojson_path'][0]

        states_names = {
			'01': "Aguascalientes",        '02': "Baja California", '03': "Baja California Sur", '04': "Campeche",
			'05': "Coahuila de Zaragoza",  '06': "Colima",          '07': "Chiapas",             '08': "Chihuahua",
			'09': "Ciudad de México",      '10': "Durango",         '11': "Guanajuato",          '12': "Guerrero",
			'13': "Hidalgo",               '14': "Jalisco",         '15': "México",              '16': "Michoacán de Ocampo",
			'17': "Morelos",               '18': "Nayarit",         '19': "Nuevo León",          '20': "Oaxaca",
			'21': "Puebla",                '22': "Querétaro",       '23': "Quintana Roo",        '24': "San Luis Potosí",
			'25': "Sinaloa",               '26': "Sonora",          '27': "Tabasco",             '28': "Tamaulipas",
			'29': "Tlaxcala",              '30': "Veracruz de Ignacio de la Llave",        '31': "Yucatán",             '32': "Zacatecas"
		}

        ## =========================================================================================
        ## =========================================================================================
        """
            CHECK OUT THE NEW DB STRUCTURE BEFORE INSERTING ANY DATA !!!
            THERE ARE NEW FIELDS TO TAKE INTO ACCOUNT !!!
        """
        ## loading files
        for index in range(1,33):
            print ("Index: ", index)

            with open(base_path + str(index) + ".geojson") as f:
                # load data
                data = json.load(f)
                feature = data['features'][0]

                # update name
                feature['properties']['NOMGEO'] = states_names[feature['properties']['CVE_ENT']]

                # data to save
                geometry = shape(feature['geometry'])

                # create polygon
                polygon = Polygon(
                    name = self.strip_accents(feature['properties']['NOMGEO'].lower()),
                    json_info = json.dumps(feature, ensure_ascii=True),
                    wkt_polygon = geometry.wkt,
                    CVE_ENT = feature['properties']['CVE_ENT']
                )

                # save to db
                polygon.save()


                # for feat in data['features']:
                #     polygon = Polygon(
                #         name      = self.strip_accents(feat['properties']['NOMGEO'].lower()),
                #         json_info = json.dumps(feat, ensure_ascii=True)
                #     )
                #
                #     polygon.save()
        ## =========================================================================================
        ## =========================================================================================


        # print ("============= "+ data['name'] + " ===================")
        # for feature in data['features']:
        #   print ("NOMGEO: ", feature['properties']['NOMGEO'])
        #   print ("CVE_ENT: ", feature['properties']['CVE_ENT'])
        #   print ("CVE_MUN: ", feature['properties']['CVE_MUN'])
        #   print ("CVEGEO: ", feature['properties']['CVEGEO'])
        #   print ("coordinates: ", feature['geometry']['coordinates'])
        #   print ("_____________________________________________")

    def strip_accents(self, s):
        return ''.join((c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn'))
